/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk;

import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.IOException;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataStore;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.AboutDialog;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.AnnotationTrackSelector;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.DataStoreSelector;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.DataTrackSelector;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.DataZoomSelector;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.EditPreferencesDialog;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.FindFeatureDialog;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.GotoDialog;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.GroupEditor;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.HelpDialog;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.LicenseDialog;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.NormalisationSelector;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.SampleEditor;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Filters.CombineFilter;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Filters.DifferencesFilter;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Filters.FeatureFilter;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Filters.PositionFilter;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Filters.StatsFilter;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Filters.WindowedDifferencesFilter;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Filters.WindowedMeanStatsFilter;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Filters.ValuesFilter;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Filters.WindowedReplicateStatsFilter;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Filters.WindowedValuesFilter;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Reports.AnnotatedListReport;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Reports.ProbeGroupReport;
import uk.ac.bbsrc.babraham.ChipMonk.Displays.ConditionTree.ConditionTree;

public class ChipMonkMenu extends JMenuBar implements ActionListener {
	
	private ChipMonkApplication application;
	private JMenu fileImport;
	private JMenu viewMenu;
	private JMenu dataMenu;
	private JMenu filterMenu;
	private JMenu reportMenu;
	private JMenuItem fileSave;
	private JMenuItem fileSaveAs;
	private JMenuItem editFindFeature;
	private JMenuItem editGotoPosition;
	private JMenuItem fileImportProbelist;
	private JMenuItem fileExportImage;
//	private JMenuItem editFindProbe;
	
	public ChipMonkMenu (ChipMonkApplication application){
		this.application = application;
		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic(KeyEvent.VK_F);
		
		JMenuItem fileNew = new JMenuItem("New project...");
		fileNew.setActionCommand("new");
		fileNew.setAccelerator(KeyStroke.getKeyStroke('N', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
		fileNew.setMnemonic(KeyEvent.VK_N);
		fileNew.addActionListener(this);
		fileMenu.add(fileNew);
		
		JMenuItem fileOpen = new JMenuItem ("Open project...");
		fileOpen.setActionCommand("open");
		fileOpen.setAccelerator(KeyStroke.getKeyStroke('O', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
		fileOpen.addActionListener(this);
		fileOpen.setMnemonic(KeyEvent.VK_O);
		fileMenu.add(fileOpen);		

		fileMenu.addSeparator();
				
		fileImport = new JMenu("Import Data");
		fileImport.setMnemonic(KeyEvent.VK_I);
		fileImport.setEnabled(false);
		
		JMenuItem fileImportNimblegen = new JMenuItem("Nimblegen...");
		fileImportNimblegen.setMnemonic(KeyEvent.VK_N);
		fileImportNimblegen.setActionCommand("import_nimblegen");
		fileImportNimblegen.addActionListener(this);
		fileImport.add(fileImportNimblegen);

		JMenuItem fileImportGeneric = new JMenuItem("Text (Generic)...");
		fileImportGeneric.setMnemonic(KeyEvent.VK_G);
		fileImportGeneric.setActionCommand("import_generic");
		fileImportGeneric.addActionListener(this);
		fileImport.add(fileImportGeneric);

		
		JMenuItem fileImportOther = new JMenuItem("Other format...");
		fileImportOther.setMnemonic(KeyEvent.VK_O);
		fileImportOther.setActionCommand("import_other");
		fileImportOther.addActionListener(this);
		fileImport.add(fileImportOther);

		// These other import methods are not fully working so I'm going to
		// comment them out of the menu.  The code behind them is still there
		// when I get more chance to work on it.
		
//		JMenuItem fileImportGenePix = new JMenuItem("GenePix...");
//		fileImportGenePix.setMnemonic(KeyEvent.VK_P);
//		fileImportGenePix.setActionCommand("import_genepix");
//		fileImportGenePix.addActionListener(this);
//		fileImport.add(fileImportGenePix);
//		
		
		fileMenu.add(fileImport);
		
		fileImportProbelist = new JMenuItem("Import Probe List...");
		fileImportProbelist.setMnemonic(KeyEvent.VK_L);
		fileImportProbelist.setActionCommand("import_probe_list");
		fileImportProbelist.addActionListener(this);
		fileImportProbelist.setEnabled(false);
		fileMenu.add(fileImportProbelist);
		
		fileMenu.addSeparator();
		
		fileExportImage = new JMenuItem("Export Current View");
		fileExportImage.setActionCommand("file_export_view");
		fileExportImage.addActionListener(this);
		fileExportImage.setMnemonic(KeyEvent.VK_E);
		fileExportImage.setEnabled(false);
		fileMenu.add(fileExportImage);
		
		
		fileMenu.addSeparator();
		
		fileSave = new JMenuItem("Save Project");
		fileSave.setActionCommand("save");
		fileSave.addActionListener(this);
		fileSave.setAccelerator(KeyStroke.getKeyStroke('S', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
		fileSave.setMnemonic(KeyEvent.VK_S);
		fileSave.setEnabled(false);
		fileMenu.add(fileSave);
		
		fileSaveAs = new JMenuItem("Save Project As...");
		fileSaveAs.setActionCommand("save_as");
		fileSaveAs.addActionListener(this);
		fileSaveAs.setMnemonic(KeyEvent.VK_A);
		fileSaveAs.setEnabled(false);
		fileMenu.add(fileSaveAs);
		
		fileMenu.addSeparator();
		
		JMenuItem fileExit = new JMenuItem("Exit");
		fileExit.setActionCommand("exit");
		fileExit.setMnemonic(KeyEvent.VK_X);
		fileExit.addActionListener(this);
		
		fileMenu.add(fileExit);
		
		add(fileMenu);
		
		JMenu editMenu = new JMenu("Edit");
		editMenu.setMnemonic(KeyEvent.VK_E);
		
		editGotoPosition = new JMenuItem("Goto Position...");
		editGotoPosition.setActionCommand("goto_position");
		editGotoPosition.setAccelerator(KeyStroke.getKeyStroke('G', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
		editGotoPosition.setMnemonic(KeyEvent.VK_G);
		editGotoPosition.addActionListener(this);
		editGotoPosition.setEnabled(false);
		editMenu.add(editGotoPosition);
		
		editFindFeature = new JMenuItem("Find Feature...");
		editFindFeature.setActionCommand("find_feature");
		editFindFeature.setAccelerator(KeyStroke.getKeyStroke('F', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
		editFindFeature.setMnemonic(KeyEvent.VK_F);
		editFindFeature.addActionListener(this);
		editFindFeature.setEnabled(false);
		editMenu.add(editFindFeature);
		
//		editFindProbe = new JMenuItem("Find probe...");
//		editFindProbe.setActionCommand("find_probe");
//		editFindProbe.setMnemonic(KeyEvent.VK_P);
//		editFindProbe.addActionListener(this);
//		editFindProbe.setEnabled(false);
//		editMenu.add(editFindProbe);
		
		editMenu.addSeparator();
		
		JMenuItem editPreferences = new JMenuItem("Preferences...");
		editPreferences.setActionCommand("edit_preferences");
		editPreferences.setMnemonic(KeyEvent.VK_R);
		editPreferences.addActionListener(this);
		editMenu.add(editPreferences);
		
		add(editMenu);
		
		
		viewMenu = new JMenu("View");
		viewMenu.setMnemonic(KeyEvent.VK_V);
		viewMenu.setEnabled(false);
		
		JCheckBoxMenuItem viewAllLabels = new JCheckBoxMenuItem("Show All Labels");
		viewAllLabels.setSelected(false);
		viewAllLabels.setActionCommand("view_all_labels");
		viewAllLabels.setAccelerator(KeyStroke.getKeyStroke('L', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
		viewAllLabels.addActionListener(this);
		viewAllLabels.setMnemonic(KeyEvent.VK_L);
		viewMenu.add(viewAllLabels);
		
		viewMenu.addSeparator();

		JMenuItem viewDataTracks = new JMenuItem("Set Data Tracks...");
		viewDataTracks.setActionCommand("view_data_tracks");
		viewDataTracks.setMnemonic(KeyEvent.VK_D);
		viewDataTracks.addActionListener(this);
		viewMenu.add(viewDataTracks);

		
		JMenuItem viewAnnotationTracks = new JMenuItem("Set Annotation Tracks...");
		viewAnnotationTracks.setActionCommand("view_annotation_tracks");
		viewAnnotationTracks.setMnemonic(KeyEvent.VK_A);
		viewAnnotationTracks.addActionListener(this);
		viewMenu.add(viewAnnotationTracks);
		
		viewMenu.addSeparator();
		
		JMenuItem viewDataSetTree = new JMenuItem("DataSet Tree...");
		viewDataSetTree.setActionCommand("view_dataset_tree");
		viewDataSetTree.setMnemonic(KeyEvent.VK_T);
		viewDataSetTree.addActionListener(this);
		viewMenu.add(viewDataSetTree);
		
		JMenuItem viewScatterPlot = new JMenuItem("Scatter Plot...");
		viewScatterPlot.setActionCommand("view_scatter_plot");
		viewScatterPlot.setMnemonic(KeyEvent.VK_S);
		viewScatterPlot.addActionListener(this);
		viewMenu.add(viewScatterPlot);

		JMenuItem viewMAPlot = new JMenuItem("MA Plot...");
		viewMAPlot.setActionCommand("view_ma_plot");
		viewMAPlot.setMnemonic(KeyEvent.VK_M);
		viewMAPlot.addActionListener(this);
		viewMenu.add(viewMAPlot);
		
		viewMenu.addSeparator();
		
		JMenuItem viewSetZoom = new JMenuItem("Set Data Zoom Level...");
		viewSetZoom.setActionCommand("view_set_zoom");
		viewSetZoom.setAccelerator(KeyStroke.getKeyStroke('Z', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
		viewSetZoom.setMnemonic(KeyEvent.VK_Z);
		viewSetZoom.addActionListener(this);
		viewMenu.add(viewSetZoom);
		
		JMenuItem viewZoomIn = new JMenuItem("Zoom In");
		viewZoomIn.setActionCommand("zoom_in");
		viewZoomIn.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0));
		viewZoomIn.setMnemonic(KeyEvent.VK_I);
		viewZoomIn.addActionListener(this);
		viewMenu.add(viewZoomIn);
		
		JMenuItem viewZoomOut = new JMenuItem("Zoom Out");
		viewZoomOut.setActionCommand("zoom_out");
		viewZoomOut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0));
		viewZoomOut.setMnemonic(KeyEvent.VK_O);
		viewZoomOut.addActionListener(this);
		viewMenu.add(viewZoomOut);
		
		JMenuItem viewMoveLeft = new JMenuItem("Move left");
		viewMoveLeft.setActionCommand("move_left");
		viewMoveLeft.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0));
		viewMoveLeft.setMnemonic(KeyEvent.VK_L);
		viewMoveLeft.addActionListener(this);
		viewMenu.add(viewMoveLeft);

		JMenuItem viewMoveRight = new JMenuItem("Move Right");
		viewMoveRight.setActionCommand("move_right");
		viewMoveRight.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0));
		viewMoveRight.setMnemonic(KeyEvent.VK_R);
		viewMoveRight.addActionListener(this);
		viewMenu.add(viewMoveRight);

		
		add(viewMenu);
		
		dataMenu = new JMenu("Data");
		dataMenu.setMnemonic(KeyEvent.VK_D);
		dataMenu.setEnabled(false);
		
		JMenuItem dataNormalise = new JMenuItem("Normalisation...");
		dataNormalise.setActionCommand("normalisation");
		dataNormalise.setMnemonic(KeyEvent.VK_N);
		dataNormalise.addActionListener(this);
		dataMenu.add(dataNormalise);
		
		JMenuItem dataSamples = new JMenuItem("Edit Samples...");
		dataSamples.setActionCommand("edit_samples");
		dataSamples.setMnemonic(KeyEvent.VK_S);
		dataSamples.addActionListener(this);
		dataMenu.add(dataSamples);
		
		JMenuItem dataGroups = new JMenuItem("Edit Groups...");
		dataGroups.setActionCommand("edit_groups");
		dataGroups.setMnemonic(KeyEvent.VK_G);
		dataGroups.addActionListener(this);
		dataMenu.add(dataGroups);
		
		
		
		add(dataMenu);
		
		filterMenu = new JMenu("Filtering");
		filterMenu.setMnemonic(KeyEvent.VK_T);
		filterMenu.setEnabled(false);
		
		JMenu filterValuesMenu = new JMenu("Filter on Values");
		filterValuesMenu.setMnemonic(KeyEvent.VK_V);
		
		JMenuItem filterValues = new JMenuItem("Individual probes...");
		filterValues.setActionCommand("filter_values_individual");
		filterValues.addActionListener(this);
		filterValues.setMnemonic(KeyEvent.VK_I);
		filterValuesMenu.add(filterValues);

		
		JMenuItem filterValuesW = new JMenuItem("Windowed average...");
		filterValuesW.setActionCommand("filter_values_windowed");
		filterValuesW.addActionListener(this);
		filterValuesW.setMnemonic(KeyEvent.VK_W);
		filterValuesMenu.add(filterValuesW);
		
		filterMenu.add(filterValuesMenu);
		
		JMenu filterDiffsMenu = new JMenu("Filter on Value Differences");
		filterDiffsMenu.setMnemonic(KeyEvent.VK_D);
		
		JMenuItem filterDiffs = new JMenuItem("Individual probes...");
		filterDiffs.setActionCommand("filter_diffs_individual");
		filterDiffs.addActionListener(this);
		filterDiffs.setMnemonic(KeyEvent.VK_I);
		filterDiffsMenu.add(filterDiffs);

		
		JMenuItem filterDiffsW = new JMenuItem("Windowed average...");
		filterDiffsW.setActionCommand("filter_diffs_windowed");
		filterDiffsW.addActionListener(this);
		filterDiffsW.setMnemonic(KeyEvent.VK_W);
		filterDiffsMenu.add(filterDiffsW);
		
		filterMenu.add(filterDiffsMenu);
		
		JMenu filterStatsMenu = new JMenu("Filter by Statistical Test");
		filterStatsMenu.setMnemonic(KeyEvent.VK_S);
		
		JMenuItem filterStats = new JMenuItem("Individual Probes...");
		filterStats.setActionCommand("filter_stats");
		filterStats.addActionListener(this);
		filterStats.setMnemonic(KeyEvent.VK_I);
		filterStatsMenu.add(filterStats);
		
		JMenuItem filterStatsW = new JMenuItem("Windowed Mean...");
		filterStatsW.setActionCommand("filter_statsw");
		filterStatsW.addActionListener(this);
		filterStatsW.setMnemonic(KeyEvent.VK_W);
		filterStatsMenu.add(filterStatsW);

		JMenuItem filterStatsR = new JMenuItem("Windowed Replicate...");
		filterStatsR.setActionCommand("filter_statsr");
		filterStatsR.addActionListener(this);
		filterStatsR.setMnemonic(KeyEvent.VK_R);
		filterStatsMenu.add(filterStatsR);
		
		filterMenu.add(filterStatsMenu);
		
		JMenuItem filterPosition = new JMenuItem("Filter by Position...");
		filterPosition.setActionCommand("filter_position");
		filterPosition.addActionListener(this);
		filterPosition.setMnemonic(KeyEvent.VK_P);
		filterMenu.add(filterPosition);
		
		JMenuItem filterFeatures = new JMenuItem("Filter by Features...");
		filterFeatures.setActionCommand("filter_features");
		filterFeatures.addActionListener(this);
		filterFeatures.setMnemonic(KeyEvent.VK_F);
		filterMenu.add(filterFeatures);
		
		JMenuItem filterCombine = new JMenuItem("Combine Existing Lists...");
		filterCombine.setActionCommand("filter_combine");
		filterCombine.addActionListener(this);
		filterCombine.setMnemonic(KeyEvent.VK_C);
		filterMenu.add(filterCombine);
		
		add(filterMenu);
		
		reportMenu = new JMenu("Reports");
		reportMenu.setMnemonic(KeyEvent.VK_R);
		reportMenu.setEnabled(false);

		JMenuItem reportAnnotated = new JMenuItem("Create Annotated Probe Report...");
		reportAnnotated.setActionCommand("report_annotated");
		reportAnnotated.addActionListener(this);
		reportAnnotated.setMnemonic(KeyEvent.VK_A);
		reportMenu.add(reportAnnotated);
		
		JMenuItem reportProbeGroup = new JMenuItem("Create Probe Group Report...");
		reportProbeGroup.setActionCommand("report_group");
		reportProbeGroup.addActionListener(this);
		reportProbeGroup.setMnemonic(KeyEvent.VK_G);
		reportMenu.add(reportProbeGroup);

		add(reportMenu);
		
		
		
		JMenu helpMenu = new JMenu("Help");
		helpMenu.setMnemonic(KeyEvent.VK_H);
		
		JMenuItem helpContents = new JMenuItem("Contents...");
		helpContents.setActionCommand("help_contents");
		helpContents.setAccelerator(KeyStroke.getKeyStroke('H', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
		helpContents.addActionListener(this);
		helpContents.setMnemonic(KeyEvent.VK_C);
		helpMenu.add(helpContents);
		
		helpMenu.addSeparator();

		JMenuItem helpLicense = new JMenuItem("License...");
		helpLicense.setActionCommand("help_license");
		helpLicense.setMnemonic(KeyEvent.VK_L);
		helpLicense.addActionListener(this);
		helpMenu.add(helpLicense);

		
		JMenuItem helpAbout = new JMenuItem("About...");
		helpAbout.setActionCommand("about");
		helpAbout.setMnemonic(KeyEvent.VK_A);
		helpAbout.addActionListener(this);
		helpMenu.add(helpAbout);
		
		add(helpMenu);
		
	}

	public void genomeLoaded() {
		fileImport.setEnabled(true);
		viewMenu.setEnabled(true);
		fileSave.setEnabled(true);
		fileSaveAs.setEnabled(true);
		editFindFeature.setEnabled(true);
		editGotoPosition.setEnabled(true);
		fileExportImage.setEnabled(true);
	}

	public void dataLoaded() {
		genomeLoaded();
		dataMenu.setEnabled(true);
		filterMenu.setEnabled(true);
		reportMenu.setEnabled(true);
		fileImportProbelist.setEnabled(true);
//		editFindProbe.setEnabled(true);
	}

	
	public void actionPerformed(ActionEvent ae) {
		String action = ae.getActionCommand();
		
		if (action.equals("exit")) {
			application.dispose();
		}
		else if (action.equals("new")) {
			application.startNewProject();
		}
		else if (action.equals("import_nimblegen")) {
			application.importNimblegen();
		}
		else if (action.equals("import_other")) {
			JOptionPane.showMessageDialog(application, "ChIPMonk can be extended to support other array formats.\n\nIf you'd like support for the format you use then please contact the program authors\nand provide them with example data files so they can write the appropriate import filters.", "Import other formats", JOptionPane.INFORMATION_MESSAGE);
		}
		else if (action.equals("import_genepix")) {
			application.importGenePix();
		}
		else if (action.equals("import_generic")) {
			application.importGeneric();
		}
		else if (action.equals("import_probe_list")) {
			application.importProbeList();
		}
		else if (action.equals("about")) {
			new AboutDialog(application);
		}
		else if (action.equals("help_license")) {
			try {
				new LicenseDialog(application);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		else if (action.equals("help_contents")) {
			try {
				new HelpDialog(application);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		else if (action.equals("zoom_in")) {
			application.chromosomeViewer().zoomIn();
		}
		else if (action.equals("zoom_out")) {
			application.chromosomeViewer().zoomOut();
		}
		else if (action.equals("move_left")) {
			application.chromosomeViewer().moveLeft();
		}
		else if (action.equals("move_right")) {
			application.chromosomeViewer().moveRight();
		}
		else if (action.equals("view_annotation_tracks")) {
			new AnnotationTrackSelector(application);
		}
		else if (action.equals("view_data_tracks")) {
			new DataTrackSelector(application);
		}
		else if (action.equals("view_dataset_tree")) {
			new ConditionTree(application);
		}
		else if (action.equals("view_scatter_plot")) {
			DataStore [] stores = application.dataViewer().getSelectedDataStores();
			if (stores.length == 0) {
				JOptionPane.showMessageDialog(application,"No data sets / groups are selected","Error",JOptionPane.ERROR_MESSAGE);
			}
			else if (stores.length == 1) {
				application.showScatterPlot(stores[0]);
			}
			else {
				DataStoreSelector s = new DataStoreSelector(stores,application);
				int si = s.getSelectedIndex();
				if (si >=0) {
					application.showScatterPlot(stores[si]);					
				}
			}
		}
		else if (action.equals("view_ma_plot")) {
			DataStore [] stores = application.dataViewer().getSelectedDataStores();
			if (stores.length == 0) {
				JOptionPane.showMessageDialog(application,"No data sets / groups are selected","Error",JOptionPane.ERROR_MESSAGE);
			}
			else if (stores.length == 1) {
				application.showMAPlot(stores[0]);
			}
			else {
				DataStoreSelector s = new DataStoreSelector(stores,application);
				int si = s.getSelectedIndex();
				if (si >=0) {
					application.showMAPlot(stores[si]);					
				}
			}
			
		}
		else if (action.equals("view_all_labels")) {
			application.chromosomeViewer().toggleLabels();
		}
		else if (action.equals("view_set_zoom")) {
			new DataZoomSelector(application);
		}
		else if (action.equals("normalisation")) {
			new NormalisationSelector(application);
		}
		else if (action.equals("edit_groups")) {
			new GroupEditor(application);
		}
		else if (action.equals("edit_samples")) {
			new SampleEditor(application);
		}
		else if (action.equals("save")) {
			application.saveProject();
		}
		else if (action.equals("save_as")) {
			application.saveProjectAs();
		}
		else if (action.equals("file_export_view")) {
			application.exportCurrentView();
		}
		else if (action.equals("open")) {
			application.loadProject();
		}
		else if (action.equals("filter_values_individual")) {
			new ValuesFilter(application);
		}
		else if (action.equals("filter_values_windowed")) {
			new WindowedValuesFilter(application);
		}
		else if (action.equals("filter_diffs_individual")) {
			new DifferencesFilter(application);
		}
		else if (action.equals("filter_diffs_windowed")) {
			new WindowedDifferencesFilter(application);
		}
		else if (action.equals("filter_stats")) {
			new StatsFilter(application);
		}
		else if (action.equals("filter_statsw")) {
			new WindowedMeanStatsFilter(application);
		}
		else if (action.equals("filter_statsr")) {
			new WindowedReplicateStatsFilter(application);
		}
		else if (action.equals("filter_position")) {
			new PositionFilter(application);
		}
		else if (action.equals("filter_features")) {
			new FeatureFilter(application);
		}
		else if (action.equals("filter_combine")) {
			new CombineFilter(application);
		}
		else if (action.equals("report_annotated")) {
			new AnnotatedListReport(application);
		}
		else if (action.equals("report_group")) {
			new ProbeGroupReport(application);
		}
		else if (action.equals("edit_preferences")) {
			new EditPreferencesDialog(application);
		}
		else if (action.equals("find_feature")) {
			new FindFeatureDialog(application);
		}
		else if (action.equals("goto_position")) {
			new GotoDialog(application);
		}
	}

}
