/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Dialogs;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.Normalisation.GlobalNormalisation;
import uk.ac.bbsrc.babraham.ChipMonk.Normalisation.LowessNormalisation;
import uk.ac.bbsrc.babraham.ChipMonk.Normalisation.MinValueNormalisation;
import uk.ac.bbsrc.babraham.ChipMonk.Normalisation.Normalisation;
import uk.ac.bbsrc.babraham.ChipMonk.Normalisation.PerProbeNormalisation;
import uk.ac.bbsrc.babraham.ChipMonk.Normalisation.ResetNormalisation;

public class NormalisationSelector extends JDialog implements ActionListener, ListSelectionListener {
	
	private ChipMonkApplication application;
	private DefaultListModel availableModel = new DefaultListModel();
	private JList availableList;
	private DefaultListModel usedModel = new DefaultListModel();
	private JList usedList;
	private JButton addButton;
	private JButton removeButton;
	private JButton upButton;
	private JButton downButton;
	
	public NormalisationSelector (ChipMonkApplication application) {
		super(application,"Select Normalisation Steps");
		setSize(600,200);
		setLocationRelativeTo(application);
		
		this.application = application;
		
		getContentPane().setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx=0;
		c.gridy=0;
		c.weightx=0.9;
		c.weighty=0.9;
		c.fill = GridBagConstraints.BOTH;
		
		JPanel availablePanel = new JPanel();
		availablePanel.setLayout(new BorderLayout());
		availablePanel.add(new JLabel("Available Modules",JLabel.CENTER),BorderLayout.NORTH);
		availableList = new JListWithTips(availableModel);
		availableList.addListSelectionListener(this);
		availablePanel.add(new JScrollPane(availableList),BorderLayout.CENTER);
		getContentPane().add(availablePanel,c);

		c.gridx++;
		c.weightx = 0.2;
		c.fill = GridBagConstraints.NONE;
		
		JPanel middlePanel = new JPanel();
		middlePanel.setLayout(new GridLayout(4,1));
		addButton = new JButton("Add");
		addButton.setActionCommand("add");
		addButton.addActionListener(this);
		addButton.setEnabled(false);
		middlePanel.add(addButton);
		
		removeButton = new JButton("Remove");
		removeButton.setActionCommand("remove");
		removeButton.addActionListener(this);
		removeButton.setEnabled(false);
		middlePanel.add(removeButton);
		
		upButton = new JButton("Move up");
		upButton.setActionCommand("up");
		upButton.addActionListener(this);
		upButton.setEnabled(false);
		middlePanel.add(upButton);
		
		downButton = new JButton("Move down");
		downButton.setActionCommand("down");
		downButton.addActionListener(this);
		downButton.setEnabled(false);
		middlePanel.add(downButton);
		
		getContentPane().add(middlePanel,c);
		
		c.gridx++;
		c.weightx = 0.9;
		c.fill = GridBagConstraints.BOTH;

		JPanel usedPanel = new JPanel();
		usedPanel.setLayout(new BorderLayout());
		usedPanel.add(new JLabel("Used Modules",JLabel.CENTER),BorderLayout.NORTH);
		usedList = new JListWithTips(usedModel);
		usedList.addListSelectionListener(this);
		usedPanel.add(new JScrollPane(usedList),BorderLayout.CENTER);
		getContentPane().add(usedPanel,c);
		
		c.gridx=0;
		c.gridy++;
		c.gridwidth=3;
		c.weighty = 0.1;
		c.fill = GridBagConstraints.NONE;

		JPanel bottomPanel = new JPanel();
		JButton cancelButton = new JButton("Cancel");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		bottomPanel.add(cancelButton);
		
		JButton okButton = new JButton("OK");
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		getRootPane().setDefaultButton(okButton);
		bottomPanel.add(okButton);
		
		getContentPane().add(bottomPanel,c);
		
		
		// Add in the modules we know about.  This is just a manually
		// curated list which will need to be updated when new 
		// normalisation modules are added.

		// Lowess isn't ready for use in production
//		availableModel.addElement(new LowessNormalisation());

		availableModel.addElement(new MinValueNormalisation());
		availableModel.addElement(new GlobalNormalisation());
		availableModel.addElement(new LowessNormalisation());
		availableModel.addElement(new PerProbeNormalisation());
		
		Normalisation [] usedModules = application.getNormalisationModules();
		
		// We start adding at 1 since the first module will always be the reset module.
		for (int i=1;i<usedModules.length;i++) {
			usedModel.addElement(usedModules[i]);
		}
		
		setVisible(true);
		
		// A bif of a hack to make sure both lists
		// resize as they should...
		availableModel.addElement("temp");
		availableModel.removeElement("temp");
		usedModel.addElement("temp");
		usedModel.removeElement("temp");
	}

	public void actionPerformed(ActionEvent ae) {
		String c = ae.getActionCommand();

		if (c.equals("add")) {
			Object [] adds = availableList.getSelectedValues();
			for (int i=0;i<adds.length;i++) {
				usedModel.addElement(adds[i]);
			}
		}
		else if (c.equals("remove")) {
			Object [] adds = usedList.getSelectedValues();
			for (int i=0;i<adds.length;i++) {
				usedModel.removeElement(adds[i]);
			}			
		}
		else if (c.equals("up")) {
			int s = usedList.getSelectedIndex();
			Object current = usedModel.elementAt(s);
			Object above = usedModel.elementAt(s-1);
			usedModel.setElementAt(current,s-1);
			usedModel.setElementAt(above,s);
			usedList.setSelectedIndex(s-1);
		}
		else if (c.equals("down")) {
			int s = usedList.getSelectedIndex();
			Object current = usedModel.elementAt(s);
			Object below = usedModel.elementAt(s+1);
			usedModel.setElementAt(current,s+1);
			usedModel.setElementAt(below,s);
			usedList.setSelectedIndex(s+1);			
		}
		else if (c.equals("cancel")) {
			setVisible(false);
			dispose();
		}
		else if (c.equals("ok")) {
			Object [] o = usedModel.toArray();
			Normalisation [] n = new Normalisation [o.length+1];
			
			// We always add a reset module to the front of the list
			n[0] = new ResetNormalisation();
			for (int i=1;i<n.length;i++) {
				n[i] = (Normalisation)o[i-1];
			}
			application.setNormalisationModules(n);
			setVisible(false);
			dispose();
		}
		
	}

	public void valueChanged(ListSelectionEvent ae) {
		// Check to see if we can add anything...
		if (availableList.getSelectedIndices().length>0) {
			addButton.setEnabled(true);
		}
		else {
			addButton.setEnabled(false);
		}
		
		//Or remove anything
		if (usedList.getSelectedIndices().length>0) {
			removeButton.setEnabled(true);
		}
		else {
			removeButton.setEnabled(false);
		}
		
		// If we can move up
		if (usedList.getSelectedIndices().length == 1 && usedList.getSelectedIndex() != 0) {
			upButton.setEnabled(true);
		}
		else {
			upButton.setEnabled(false);
		}
		
		// If we can move down
		if (usedList.getSelectedIndices().length == 1 && usedList.getSelectedIndex() != usedList.getLastVisibleIndex()) {
			downButton.setEnabled(true);
		}
		else {
			downButton.setEnabled(false);
		}
	}
	
	private class JListWithTips extends JList {
		
		public JListWithTips (ListModel model) {
			super(model);
		}
		public String getToolTipText (MouseEvent e) {
			Normalisation n = (Normalisation)getModel().getElementAt(locationToIndex(e.getPoint()));
			return n.longDescription();
		}
	}

}
