/**
 * Copyright 2007 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Displays;

import java.awt.Component;
import java.text.DecimalFormat;
import java.text.NumberFormat;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;

public class SmallDoubleCellRenderer extends DefaultTableCellRenderer {

	
	// This class is used to render very small double values correctly
	// whilst still allowing them to be sorted.  The default JTable
	// renderers make values below 0.001 come out as 0, which isn't
	// ideal when dealing with p/q values.
	
	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus,int row,int column) {
		
		final NumberFormat format = new DecimalFormat("0.##E0");
		
		Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
		if (value instanceof Double) {
			((JLabel)c).setText(format.format(((Double)value).doubleValue()));
		}
		
		return c;
	}

}
