/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Normalisation;

import java.util.Enumeration;
import java.util.Vector;

import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataCollection;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataSet;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;

public class ResetNormalisation implements Normalisation, Runnable {

	private Vector listeners = new Vector();
	private DataCollection data;
	
	public void normalise(DataCollection d) {
		data = d;

		Thread t = new Thread(this);
		t.start();
	}

	public void addNormalisationProgressListener(NormalisationProgressListener l) {
		listeners.add(l);
	}

	public boolean isSuitable(DataCollection d) {
		return true;
	}

	public void run() {		
		Probe [] probes = data.probeSet().getAllProbes();
		DataSet [] sets = data.getAllDataSets();
		
		for (int s=0;s<sets.length;s++) {
			Enumeration e = listeners.elements();
			while (e.hasMoreElements()) {
				NormalisationProgressListener l = (NormalisationProgressListener)e.nextElement();
				l.updateNormalisationProgress("Resetting Normalisation for "+sets[s].name());
			}
			for (int p=0;p<probes.length;p++) {
				sets[s].setControlForProbe(probes[p],sets[s].getRawControlValueForProbe(probes[p]));
				sets[s].setSignalForProbe(probes[p],sets[s].getRawSignalValueForProbe(probes[p]));
			}
			try {
				Thread.sleep(20);
			} catch (InterruptedException e1) {
			}
		}
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((NormalisationProgressListener)e.nextElement()).normalisationComplete();
		}
	}

	public String shortDescription() {
		return "Reset normalisation";
	}

	public String longDescription() {
		return "Resets the normalised values to their original raw values";
	}


}
